#ifndef RT_VK_GPU_H
#define RT_VK_GPU_H

#include <volk/volk.h>

#define VMA_STATIC_VULKAN_FUNCTIONS 0
#define VMA_DYNAMI_VULKAN_FUNCTIONS 0
#include <vma/vk_mem_alloc.h>

#include "runtime/renderer_api.h"

/* Minimum supported value of g_gpu.max_frames_in_flight */
#define RT_VK_MIN_SUPPORTED_FRAMES_IN_FLIGHT 2

/* Maximum supported number of frames in flight.
 * The actually configured value is contained in g_gpu. */
#define RT_VK_MAX_SUPPORTED_FRAMES_IN_FLIGHT 3

#ifdef _WIN32
struct HINSTANCE__;
struct HWND__;
#elif defined(RT_USE_XLIB)
struct _XDisplay;
#endif

typedef struct {
#ifdef _WIN32
    struct HINSTANCE__ *hInstance;
    struct HWND__ *hWnd;
#elif defined(RT_USE_XLIB)
    struct _XDisplay *display;
    unsigned long window;
#endif
} rt_native_window;

typedef struct {
    VkInstance instance;
    VkDebugUtilsMessengerEXT messenger;
    VkAllocationCallbacks *alloc_cb;
    VkPhysicalDevice phys_device;
    VkDevice device;
    VkSurfaceKHR surface;
    VkQueue graphics_queue;
    VkQueue compute_queue;
    VkQueue present_queue;
    VkQueue transfer_queue;
    uint32_t graphics_family;
    uint32_t compute_family;
    uint32_t present_family;
    uint32_t transfer_family;

    rt_native_window native_window;

    VkPhysicalDeviceDescriptorIndexingProperties descriptor_indexing_props;
    VkPhysicalDeviceProperties phys_device_props;
    VkPhysicalDeviceDescriptorIndexingFeatures descriptor_indexing_features;
    VkPhysicalDeviceFeatures phys_device_features;

    VmaAllocator allocator;

    unsigned int max_frames_in_flight;
    unsigned int current_frame_id;
} rt_vk_gpu;

#ifndef RT_VK_DONT_DEFINE_GPU_GLOBAL
extern rt_vk_gpu g_gpu;
#endif

/* Helper functions */

VkFormat rtPixelFormatToVkFormat(rt_pixel_format format);

VkSampleCountFlagBits rtSampleCountToFlags(unsigned int count);

VkQueue rtGetQueue(rt_gpu_queue queue);

#endif
